package de.tudarmstadt.informatik.hostage.protocol;

import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;

import de.tudarmstadt.informatik.hostage.commons.HelperUtils;
import de.tudarmstadt.informatik.hostage.wrapper.Packet;


/**
 * HTTP protocol
 * @author Wulf Pfeiffer
 */
public class HTTP implements Protocol {
	
	@Override
	public int getDefaultPort() {
		return 80;
	}

	@Override
	public TALK_FIRST whoTalksFirst() {
		return TALK_FIRST.CLIENT;
	}
	
	@Override
	public List<Packet> processMessage(Packet packet) {
		String request = null;
		if (packet != null) {
			request = packet.toString();
		}
		List<Packet> response = new ArrayList<Packet>();
		this.request = request;

		if(!request.contains(httpVersion)){
			response.add(buildPacket(c505, ""));
		} else if(request.contains(get)) {
			response.add(buildPacket(c200, get));
		} else if(request.contains(head)) {
			response.add(buildPacket(c200, head));
		} else if(request.contains(trace)){
			response.add(buildPacket(c200, trace));
		} else if(request.contains(options)){
			response.add(buildPacket(c400, options));
		} else if(request.contains(post)){
			response.add(buildPacket(c200, post));
		} else if(request.contains(put)){
			response.add(buildPacket(c400, put));
		} else if(request.contains(delete)){
			response.add(buildPacket(c200, delete));
		} else if(request.contains(connect)){
			response.add(buildPacket(c400, connect));
		} else {
			response.add(buildPacket(c400, ""));
		}
		return response;
	}

	@Override
	public boolean isClosed() {
		return true;
	}

	@Override
	public boolean isSecure() {
		return false;
	}

	@Override
	public Class<String> getType() {
		return String.class;
	}

	@Override
	public String toString() {
		return "HTTP";
	}

	/**
	 * Builds a html response that can be sent
	 * @param code response code that was determined
	 * @param type request type that was sent by the client
	 * @return the html response
	 */
	private Packet buildPacket(String code, String type) {
		String doc = "";
		if(type.equals(get)) doc = htmlDoc;
		else if(type.equals(head) || type.equals(delete)) doc = "";
		else if(type.equals(trace)) doc = request;
		else doc = errorHtmlPrefix + " " + code + errorHtmlSuffix;

		return new Packet(httpVersion + " " + code + headerPrefix + doc.length() + headerSuffix + doc);
	}
	
	/**
	 * Get the current time in html header format.
	 * @return the formatted server time.
	 */
	private static String getServerTime() {
	    Calendar calendar = Calendar.getInstance();
	    SimpleDateFormat dateFormat = new SimpleDateFormat(
	        "EEE, dd MMM yyyy HH:mm:ss z", Locale.US);
	    dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
	    return dateFormat.format(calendar.getTime());
	}

	/** Whole request that was sent by the client */
	private String request	= "";
	//version stuff
	private static String[][][] possibleHttpVersions = {
		{{"Apache/2.0."},{"28","32","35","36","39","40","42","43","44","45","46","47","48","49","50","51","52","53","54","55","58","59","61","63","64","65"}},
		{{"Apache/2.2."},{"0","2","3","4","6","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25"}},
		{{"Apache/2.3."},{"4","5","6","8","10","11","12","14","15","16"}},
		{{"Apache/2.4."},{"1","2","3","4","6"}}
	};
	private static String initServerVersion() {
		SecureRandom rndm = new SecureRandom();
		int majorVersion = rndm.nextInt(possibleHttpVersions.length);
		return possibleHttpVersions[majorVersion][0][0] + possibleHttpVersions[majorVersion][1][rndm.nextInt(possibleHttpVersions[majorVersion][1].length)];
	}
	private String httpVersion	= "HTTP/1.1";
	private static String serverVersion = initServerVersion();
	private static String htmlDocContent = HelperUtils.getRandomString(32, false);
	//request codes
	private String options 	= "OPTIONS";
	private String get 		= "GET";
	private String head 	= "HEAD";
	private String post		= "POST";
	private String put		= "PUT";
	private String delete	= "DELETE";
	private String trace	= "TRACE";
	private String connect	= "CONNECT";
	private String c200 	= "200 OK\r\n";
	private String c400 	= "400 Bad Request\r\n";
	private String c505 	= "505 HTTP Version not supported\r\n";
	
	//html header pre and suffix
	private static final String headerPrefix =				
			"Date: " + getServerTime() + "\r\n" +
			"Server: " + serverVersion + " \r\n" +
			"Vary: Accept-Encoding\r\n" +
			"Content-Length: ";
	private String headerSuffix =
			"\r\n" +	
			"Keep-Alive: timeout=5, max=100\r\n" +
			"Connection: Keep-Alive\r\n" +
			"Content-Type: text/html\r\n" +
			"\r\n";
	//html website
	private String htmlDoc = 
			"<!doctype html>\n" +
			"<html lang=\"en\">\n" +
			"<head>\n" +
			"<meta charset=\"UTF-8\">\n" +
			"<title>" + htmlDocContent + "</title>\n" +
			"<body>" + htmlDocContent + "</body>\n" +
			"</head>\n" +
			"</html>";
	//html error pre and suffix
	private String errorHtmlPrefix =
			"<!doctype html>\n" +
			"<html lang=\"en\">\n" +
			"<head>\n" +
			"<meta charset=\"UTF-8\">\n" +
			"<title>";
	private String errorHtmlSuffix =
			"</title>\n" +
			"</head>\n" +
			"</html>";
	
	public static String getHtmlDocContent() {
		return htmlDocContent;
	}

	public static void setHtmlDocContent(String htmlDocContent) {
		HTTP.htmlDocContent = htmlDocContent;
	}
}
